==> hermes-2.5.1.30.tar.gz: hermes-2.5.1.30/app/controllers/hermes/api/dmo/housing_applications_controller.rb <==
# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des methodes
module Hermes
  module Api
    module Dmo
      class HousingApplicationsController < Hermes::Api::Dmo::ApplicationController
        hermes_tools 'aggregations'
        require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ]

        ACCEPTABLE_PARAMS = %w[
          ids
          housing_request_id
          housing_offer_id
          epci
          max_date
          applicated_at
          effort_rate
          status
          status_updated_at
          visit_conducted_date
          visit_status
          visit_status_updated_at
          lessor_event_informations
          last_status_update_date
          rank
          profile_epci_score
          eligible_position
          applicated_position
          effort_rate
          score_id
          is_active
        ]

        REGEX_PARAMS = %w[
        ]

        MUTATE_PARAMS = %i[
          housing_request_id
          housing_offer_id
          applicant_id
          nud
          birth_date
          epci
          max_date
          status
          visit_status
          visit_conducted_date
          lessor_event_informations
          last_status_update_date
          rank
        ]

        mutate_params_only(*MUTATE_PARAMS)

        def index
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}

          options[:join_housing_offer]= true if params[:join_housing_offer]

          paginate(
            index_aggregation,
            each_serializer: Hermes::HousingApplicationLightSerializer,
            **options
          )
        end

        def show
          @options[:join_housing_offer]= true if params[:join_housing_offer]
          super
        end

        def update_status
          return render_json_error(:unprocessable_entity, :no_status) unless params["data"]["attributes"]["status"]
          housing_request_ids = Hermes::HousingRequest.where(is_active: true, unique_numbers: params["data"]["attributes"]["unique_number"]).pluck(:id)
          return render_json_error(:unprocessable_entity, :unique_number_not_found) if housing_request_ids.empty?
          housing_offer = Hermes::HousingOffer.find_by(external_ref: params["data"]["attributes"]["external_ref"])
          return render_json_error(:unprocessable_entity, :housing_offer_not_found) if !housing_offer
          applications = Hermes::HousingApplication.where(housing_offer_id: housing_offer.id, housing_request: {"$in": housing_request_ids}).order_by(updated_at: :desc)
          if applications.count > 1
            Log.perform_later(level: 'error', domain: 'api', housing_application_id: applications.first.id.to_s, code: "UPDATE_STATUS_ERR", action: "update_status : --  Too much applications", data: params["data"]["attributes"].as_json );
            return render_json_error(:unprocessable_entity, :multiple_applications_found);
          end
          @instance = applications.first
          return render_json_error(:not_found, :object_not_found) unless @instance
          unless @instance.reload && @instance.writable_by?(@user)
            Log.perform_later(level: 'error', domain: 'api', housing_application_id: @instance.id.to_s, code: "UPDATE_STATUS_ERR", action: "update_status -- User not writer", data: params["data"]["attributes"].as_json );
            return render_json_error(:unprocessable_entity, :not_writers)
          end
          
          old_status = @instance.status&.to_s
          new_status = params["data"]["attributes"]["status"]&.to_s
          mapping = {
            "INSCRIPTION CAL" => ['TRANSMIS BAILLEUR', 'INSCRIPTION CAL', 'RETENUE EN CAL SOUS RESERVE', 'RETENUE EN CAL', 'RETENUE EN CAL SOUS CONDITIONS', 'VISITE PROPOSEE', 'VISITE EFFECTUEE'],
            "RETENUE EN CAL SOUS RESERVE" => ['TRANSMIS BAILLEUR', 'INSCRIPTION CAL', 'RETENUE EN CAL SOUS RESERVE', 'RETENUE EN CAL', 'RETENUE EN CAL SOUS CONDITIONS', 'VISITE PROPOSEE', 'VISITE EFFECTUEE'],
            "RETENUE EN CAL" => ['TRANSMIS BAILLEUR', 'INSCRIPTION CAL', 'RETENUE EN CAL SOUS RESERVE', 'RETENUE EN CAL', 'RETENUE EN CAL SOUS CONDITIONS', 'VISITE PROPOSEE', 'VISITE EFFECTUEE'],
            "RETENUE EN CAL SOUS CONDITIONS" => ['TRANSMIS BAILLEUR', 'INSCRIPTION CAL', 'RETENUE EN CAL SOUS RESERVE', 'RETENUE EN CAL', 'RETENUE EN CAL SOUS CONDITIONS', 'VISITE PROPOSEE', 'VISITE EFFECTUEE'],
            "REFUSEE EN CAL" => ['TRANSMIS BAILLEUR', 'INSCRIPTION CAL', 'RETENUE EN CAL SOUS RESERVE', 'RETENUE EN CAL', 'RETENUE EN CAL SOUS CONDITIONS', 'VISITE PROPOSEE', 'VISITE EFFECTUEE'],
            "DESISTEMENT" => ['TRANSMIS BAILLEUR', 'INSCRIPTION CAL', 'RETENUE EN CAL SOUS RESERVE', 'RETENUE EN CAL', 'RETENUE EN CAL SOUS CONDITIONS', 'VISITE PROPOSEE', 'VISITE EFFECTUEE'],
            "VISITE PROPOSEE" => ['TRANSMIS BAILLEUR', 'INSCRIPTION CAL', 'RETENUE EN CAL SOUS RESERVE', 'RETENUE EN CAL', 'RETENUE EN CAL SOUS CONDITIONS', 'VISITE PROPOSEE', 'VISITE EFFECTUEE'],
            "VISITE EFFECTUEE" => ['TRANSMIS BAILLEUR', 'INSCRIPTION CAL', 'RETENUE EN CAL SOUS RESERVE', 'RETENUE EN CAL', 'RETENUE EN CAL SOUS CONDITIONS', 'VISITE PROPOSEE', 'VISITE EFFECTUEE'],
            "LOGE" => ['TRANSMIS BAILLEUR', 'INSCRIPTION CAL', 'RETENUE EN CAL SOUS RESERVE', 'RETENUE EN CAL', 'RETENUE EN CAL SOUS CONDITIONS', 'VISITE PROPOSEE', 'VISITE EFFECTUEE']
          }

          not_valid_change = new_status && old_status && mapping[new_status].present? && mapping[new_status].include?(old_status)
          unless not_valid_change
            Log.perform_later(level: 'error', domain: 'api', housing_application_id: @instance.id.to_s, code: "UPDATE_STATUS_ERR", action: "update_status -- cannot change #{old_status} to #{new_status}", data: params["data"]["attributes"].as_json )
            return render_json_error(:unprocessable_entity, :cannot_change_to_status_from_current_status)
          end
          check_params_and_update
        end

        def attribution
          # https://www.wrike.com/open.htm?id=461622137
          @offer = Hermes::HousingOffer.find mutate_valid_params[:housing_offer_id]
          return render_json_error(:not_found, :object_not_found) unless @offer
          return render_json_error(:unprocessable_entity, :not_requalified) unless @offer.requalification

          if applicant_id = mutate_valid_params[:applicant_id]
            @applicant = Hermes::Applicant.find applicant_id
          elsif uniq_numb = mutate_valid_params[:nud]
            birth_date = mutate_valid_params[:birth_date]
            @applicant = Hermes::Applicant.find_or_create_by_nud(uniq_numb, birth_date)
          end

          if @applicant.errors.present?
            return render_json_error(:unprocessable_entity, :invalid_birth_date) if @applicant.errors.messages[:birth_date]
            return render_json_error(:not_found, :invalid_nud) if @applicant.errors.messages[:nud]
          end

          @applicant.housing_requests.each do |hr|
            hr.housing_applications.update_all(status: 'ANNULEE', is_active: false) # (a) si le demandeur a une candidature en cours elle est annulée
            hr.update(request_status: 'COMPLETE') # (b) passer les demandes en logé
          end

          @new_request = Hermes::HousingRequest.create(main_applicant: @applicant)
          @application = Hermes::HousingApplication.create(
            housing_offer: @offer,
            housing_request: @new_request,
            applicated_at: DateTime.current
          )
          @offer.update(lessor_status: 'LOUE', offer_status: "RETIRE PAR LE BAILLEUR") # (b) et l'offre en attribuée + (c) l'offre est comptabilisé dans les objectifs "attribués"
          @application.reload

          render(jsonapi: @application, serializer: @serializer, status: :created)
        end
      end
    end
  end
end

==> hermes-2.5.1.36.tar.gz: hermes-2.5.1.36/app/controllers/hermes/api/dmo/housing_applications_controller.rb <==
# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des methodes
module Hermes
  module Api
    module Dmo
      class HousingApplicationsController < Hermes::Api::Dmo::ApplicationController
        hermes_tools 'aggregations'
        require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ]

        ACCEPTABLE_PARAMS = %w[
          ids
          housing_request_id
          housing_offer_id
          epci
          max_date
          applicated_at
          effort_rate
          status
          status_updated_at
          visit_conducted_date
          visit_status
          visit_status_updated_at
          lessor_event_informations
          last_status_update_date
          rank
          profile_epci_score
          eligible_position
          applicated_position
          effort_rate
          score_id
          is_active
          als_id
        ]

        REGEX_PARAMS = %w[
        ]

        MUTATE_PARAMS = %i[
          housing_request_id
          housing_offer_id
          applicant_id
          nud
          birth_date
          epci
          max_date
          status
          visit_status
          visit_conducted_date
          lessor_event_informations
          last_status_update_date
          rank
        ]

        mutate_params_only(*MUTATE_PARAMS)

        def index
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}

          options[:join_housing_offer]= true if params[:join_housing_offer]

          paginate(
            index_aggregation,
            each_serializer: Hermes::HousingApplicationLightSerializer,
            **options
          )
        end

        def show
          @options[:join_housing_offer]= true if params[:join_housing_offer]
          super
        end

        def update_status
          return render_json_error(:unprocessable_entity, :no_status) unless params["data"]["attributes"]["status"]
          housing_request_ids = Hermes::HousingRequest.where(is_active: true, unique_numbers: params["data"]["attributes"]["unique_number"]).pluck(:id)
          return render_json_error(:unprocessable_entity, :unique_number_not_found) if housing_request_ids.empty?
          housing_offer = Hermes::HousingOffer.find_by(external_ref: params["data"]["attributes"]["external_ref"])
          return render_json_error(:unprocessable_entity, :housing_offer_not_found) if !housing_offer
          applications = Hermes::HousingApplication.where(housing_offer_id: housing_offer.id, housing_request: {"$in": housing_request_ids}).order_by(updated_at: :desc)
          if applications.count > 1
            Log.perform_later(level: 'error', domain: 'api', housing_application_id: applications.first.id.to_s, code: "UPDATE_STATUS_ERR", action: "update_status : --  Too much applications", data: params["data"]["attributes"].as_json );
            return render_json_error(:unprocessable_entity, :multiple_applications_found);
          end
          @instance = applications.first
          return render_json_error(:not_found, :object_not_found) unless @instance
          unless @instance.reload && @instance.writable_by?(@user)
            Log.perform_later(level: 'error', domain: 'api', housing_application_id: @instance.id.to_s, code: "UPDATE_STATUS_ERR", action: "update_status -- User not writer", data: params["data"]["attributes"].as_json );
            return render_json_error(:unprocessable_entity, :not_writers)
          end
          
          old_status = @instance.status&.to_s
          new_status = params["data"]["attributes"]["status"]&.to_s

          list_old_status = Hermes::Statuses::HousingApplication.list.values.flatten;
          mapping = {
            "INSCRIPTION CAL" => list_old_status,
            "RETENUE EN CAL SOUS RESERVE" => list_old_status,
            "RETENUE EN CAL" => list_old_status,
            "RETENUE EN CAL SOUS CONDITIONS" => list_old_status,
            "REFUSEE EN CAL" => list_old_status,
            "DESISTEMENT" => list_old_status,
            "VISITE PROPOSEE" => list_old_status,
            "VISITE EFFECTUEE" => list_old_status,
            "LOGE" => list_old_status
          }

          not_valid_change = new_status && old_status && mapping[new_status].present? && mapping[new_status].include?(old_status)
          unless not_valid_change
            Log.perform_later(level: 'error', domain: 'api', housing_application_id: @instance.id.to_s, code: "UPDATE_STATUS_ERR", action: "update_status -- cannot change #{old_status} to #{new_status}", data: params["data"]["attributes"].as_json )
            return render_json_error(:unprocessable_entity, :cannot_change_to_status_from_current_status)
          end
          check_params_and_update
        end

        def attribution
          # https://www.wrike.com/open.htm?id=461622137
          @offer = Hermes::HousingOffer.find mutate_valid_params[:housing_offer_id]
          return render_json_error(:not_found, :object_not_found) unless @offer
          return render_json_error(:unprocessable_entity, :not_requalified) unless @offer.requalification

          if applicant_id = mutate_valid_params[:applicant_id]
            @applicant = Hermes::Applicant.find applicant_id
          elsif uniq_numb = mutate_valid_params[:nud]
            birth_date = mutate_valid_params[:birth_date]
            @applicant = Hermes::Applicant.find_or_create_by_nud(uniq_numb, birth_date)
          end

          if @applicant.errors.present?
            return render_json_error(:unprocessable_entity, :invalid_birth_date) if @applicant.errors.messages[:birth_date]
            return render_json_error(:not_found, :invalid_nud) if @applicant.errors.messages[:nud]
          end

          @applicant.housing_requests.each do |hr|
            hr.housing_applications.update_all(status: 'ANNULEE', is_active: false) # (a) si le demandeur a une candidature en cours elle est annulée
            hr.update(request_status: 'COMPLETE') # (b) passer les demandes en logé
          end

          @new_request = Hermes::HousingRequest.create(main_applicant: @applicant)
          @application = Hermes::HousingApplication.create(
            housing_offer: @offer,
            housing_request: @new_request,
            applicated_at: DateTime.current
          )
          @offer.update(lessor_status: 'LOUE', offer_status: "RETIRE PAR LE BAILLEUR") # (b) et l'offre en attribuée + (c) l'offre est comptabilisé dans les objectifs "attribués"
          @application.reload

          render(jsonapi: @application, serializer: @serializer, status: :created)
        end
      end
    end
  end
end
